@@ -9,17 +9,13 @@
 
 module Assistant.WebApp.DashBoard where
 
-import Assistant.Common
-import Assistant.WebApp
-import Assistant.WebApp.Types
+import Assistant.WebApp.Common
 import Assistant.WebApp.Utility
-import Assistant.WebApp.SideBar
+import Assistant.WebApp.RepoList
 import Assistant.WebApp.Notifications
-import Assistant.WebApp.Configurators
-import Assistant.DaemonStatus
 import Assistant.TransferQueue
+import Assistant.DaemonStatus
 import Utility.NotificationBroadcaster
-import Utility.Yesod
 import Logs.Transfer
 import Utility.Percentage
 import Utility.DataUnits
@@ -27,31 +23,27 @@ import Types.Key
 import qualified Remote
 import qualified Git
 
-import Yesod
 import Text.Hamlet
 import qualified Data.Map as M
 import Control.Concurrent
 
-{- A display of currently running and queued transfers.
- -
- - Or, if there have never been any this run, an intro display. -}
+{- A display of currently running and queued transfers. -}
 transfersDisplay :: Bool -> Widget
 transfersDisplay warnNoScript = do
-	webapp <- lift getYesod
-	current <- lift $ M.toList <$> getCurrentTransfers
-	queued <- liftIO $ getTransferQueue $ transferQueue webapp
+	webapp <- liftH getYesod
+	current <- liftH $ M.toList <$> getCurrentTransfers
+	queued <- take 10 <$> liftAssistant getTransferQueue
 	autoUpdate ident NotifierTransfersR (10 :: Int) (10 :: Int)
 	let transfers = simplifyTransfers $ current ++ queued
-	if null transfers
-		then ifM (lift $ showIntro <$> getWebAppState)
-			( introDisplay ident
-			, $(widgetFile "dashboard/transfers")
-			)
-		else $(widgetFile "dashboard/transfers")
-	where
-		ident = "transfers"
-		isrunning info = not $
-			transferPaused info || isNothing (startedTime info)
+	let transfersrunning = not $ null transfers
+	scanrunning <- if transfersrunning
+		then return False
+		else liftAssistant $ transferScanRunning <$> getDaemonStatus
+	$(widgetFile "dashboard/transfers")
+  where
+	ident = "transfers"
+	isrunning info = not $
+		transferPaused info || isNothing (startedTime info)
 
 {- Simplifies a list of transfers, avoiding display of redundant
  - equivilant transfers. -}
@@ -68,39 +60,40 @@ simplifyTransfers (v@(t1, _):r@((t2, _):l))
  -
  - Note that the head of the widget is not included, only its
  - body is. To get the widget head content, the widget is also 
- - inserted onto the getHomeR page.
+ - inserted onto the getDashboardR page.
  -}
 getTransfersR :: NotificationId -> Handler RepHtml
 getTransfersR nid = do
-	waitNotifier transferNotifier nid
+	waitNotifier getTransferBroadcaster nid
 
-	page <- widgetToPageContent $ transfersDisplay False
-	hamletToRepHtml $ [hamlet|^{pageBody page}|]
+	p <- widgetToPageContent $ transfersDisplay False
+	hamletToRepHtml $ [hamlet|^{pageBody p}|]
 
 {- The main dashboard. -}
 dashboard :: Bool -> Widget
 dashboard warnNoScript = do
-	sideBarDisplay
-	let content = transfersDisplay warnNoScript
+	let repolist = repoListDisplay $
+		mainRepoSelector { nudgeAddMore = True }
+	let transferlist = transfersDisplay warnNoScript
 	$(widgetFile "dashboard/main")
 
-getHomeR :: Handler RepHtml
-getHomeR = ifM (inFirstRun)
-	( redirect ConfigR
-	, bootstrap (Just DashBoard) $ dashboard True
+getDashboardR :: Handler RepHtml
+getDashboardR = ifM (inFirstRun)
+	( redirect ConfigurationR
+	, page "" (Just DashBoard) $ dashboard True
 	)
 
 {- Used to test if the webapp is running. -}
-headHomeR :: Handler ()
-headHomeR = noop
+headDashboardR :: Handler ()
+headDashboardR = noop
 
-{- Same as HomeR, except no autorefresh at all (and no noscript warning). -}
+{- Same as DashboardR, except no autorefresh at all (and no noscript warning). -}
 getNoScriptR :: Handler RepHtml
-getNoScriptR = bootstrap (Just DashBoard) $ dashboard False
+getNoScriptR = page "" (Just DashBoard) $ dashboard False
 
-{- Same as HomeR, except with autorefreshing via meta refresh. -}
+{- Same as DashboardR, except with autorefreshing via meta refresh. -}
 getNoScriptAutoR :: Handler RepHtml
-getNoScriptAutoR = bootstrap (Just DashBoard) $ do
+getNoScriptAutoR = page "" (Just DashBoard) $ do
 	let ident = NoScriptR
 	let delayseconds = 3 :: Int
 	let this = NoScriptAutoR
@@ -124,24 +117,21 @@ getFileBrowserR = whenM openFileBrowser $ redirectBack
  - blocking the response to the browser on it. -}
 openFileBrowser :: Handler Bool
 openFileBrowser = do
-	path <- runAnnex (error "no configured repository") $
-		fromRepo Git.repoPath
+	path <- liftAnnex $ fromRepo Git.repoPath
 	ifM (liftIO $ inPath cmd <&&> inPath cmd)
 		( do
 			void $ liftIO $ forkIO $ void $
 				boolSystem cmd [Param path]
 			return True
 		, do
-			clearUltDest
-			setUltDest $ "file://" ++ path
-			void $ redirectUltDest HomeR
+			void $ redirect $ "file://" ++ path
 			return False
 		)
-	where
+  where
 #ifdef darwin_HOST_OS
-		cmd = "open"
+	cmd = "open"
 #else
-		cmd = "xdg-open"
+	cmd = "xdg-open"
 #endif
 
 {- Transfer controls. The GET is done in noscript mode and redirects back
