@@ -5,19 +5,14 @@
  - Licensed under the GNU AGPL version 3 or higher.
  -}
 
-{-# LANGUAGE TypeFamilies, QuasiQuotes, MultiParamTypeClasses, TemplateHaskell, OverloadedStrings, RankNTypes #-}
+{-# LANGUAGE QuasiQuotes, TemplateHaskell, OverloadedStrings #-}
 
 module Assistant.WebApp.Documentation where
 
-import Assistant.Common
-import Assistant.WebApp
-import Assistant.WebApp.Types
-import Assistant.WebApp.SideBar
+import Assistant.WebApp.Common
 import Assistant.Install (standaloneAppBase)
-import Utility.Yesod
 import Build.SysConfig (packageversion)
-
-import Yesod
+import BuildFlags
 
 {- The full license info may be included in a file on disk that can
  - be read in and displayed. -}
@@ -27,9 +22,7 @@ licenseFile = do
 	return $ (</> "LICENSE") <$> base
 
 getAboutR :: Handler RepHtml
-getAboutR = bootstrap (Just About) $ do
-	sideBarDisplay
-	setTitle "About git-annex"
+getAboutR = page "About git-annex" (Just About) $ do
 	builtinlicense <- isJust <$> liftIO licenseFile
 	$(widgetFile "documentation/about")
 
@@ -38,8 +31,12 @@ getLicenseR = do
 	v <- liftIO licenseFile
 	case v of
 		Nothing -> redirect AboutR
-		Just f -> bootstrap (Just About) $ do
+		Just f -> customPage (Just About) $ do
 			-- no sidebar, just pages of legalese..
 			setTitle "License"
 			license <- liftIO $ readFile f
 			$(widgetFile "documentation/license")
+
+getRepoGroupR :: Handler RepHtml
+getRepoGroupR = page "About repository groups" (Just About) $ do
+	$(widgetFile "documentation/repogroup")
