@@ -12,11 +12,12 @@ import qualified Annex
 import Types.Backend
 import Types.Key
 import Types.KeySource
+import Utility.ExternalSHA
 
 import qualified Build.SysConfig as SysConfig
 import Data.Digest.Pure.SHA
 import qualified Data.ByteString.Lazy as L
-import System.Process
+import Data.Char
 
 type SHASize = Int
 
@@ -34,6 +35,7 @@ genBackend size = Just $ Backend
 	{ name = shaName size
 	, getKey = keyValue size
 	, fsckKey = Just $ checkKeyChecksum size
+	, canUpgradeKey = Just $ needsUpgrade
 	}
 
 genBackendE :: SHASize -> Maybe Backend
@@ -53,28 +55,11 @@ shaNameE size = shaName size ++ "E"
 shaN :: SHASize -> FilePath -> Integer -> Annex String
 shaN shasize file filesize = do
 	showAction "checksum"
-	case shaCommand shasize filesize of
-		Left sha -> liftIO $ sha <$> L.readFile file
-		Right command -> liftIO $ parse command . lines <$>
-			readsha command (toCommand [File file])
-	where
-		parse command [] = bad command
-		parse command (l:_)
-			| null sha = bad command
-			| otherwise = sha
-			where
-				sha = fst $ separate (== ' ') l
-		bad command = error $ command ++ " parse error"
-		{- sha commands output the filename, so need to set fileEncoding -}
-		readsha command args =
-			withHandle StdoutHandle createProcessSuccess p $ \h -> do
-				fileEncoding h
-				output  <- hGetContentsStrict h
-				hClose h
-				return output
-			where
-				p = (proc command args)
-					{ std_out = CreatePipe }
+	liftIO $ case shaCommand shasize filesize of
+		Left sha -> sha <$> L.readFile file
+		Right command ->
+			either error return 
+				=<< externalSHA command shasize file
 
 shaCommand :: SHASize -> Integer -> Either (L.ByteString -> String) String
 shaCommand shasize filesize
@@ -84,14 +69,14 @@ shaCommand shasize filesize
 	| shasize == 384 = use SysConfig.sha384 sha384
 	| shasize == 512 = use SysConfig.sha512 sha512
 	| otherwise = error $ "bad sha size " ++ show shasize
-	where
-		use Nothing sha = Left $ showDigest . sha
-		use (Just c) sha
-			-- use builtin, but slower sha for small files
-			-- benchmarking indicates it's faster up to
-			-- and slightly beyond 50 kb files
-			| filesize < 51200 = use Nothing sha
-			| otherwise = Right c
+  where
+	use Nothing sha = Left $ showDigest . sha
+	use (Just c) sha
+		{- use builtin, but slower sha for small files
+		 - benchmarking indicates it's faster up to
+		 - and slightly beyond 50 kb files -}
+		| filesize < 51200 = use Nothing sha
+		| otherwise = Right c
 
 {- A key is a checksum of its contents. -}
 keyValue :: SHASize -> KeySource -> Annex (Maybe Key)
@@ -109,23 +94,21 @@ keyValue shasize source = do
 {- Extension preserving keys. -}
 keyValueE :: SHASize -> KeySource -> Annex (Maybe Key)
 keyValueE size source = keyValue size source >>= maybe (return Nothing) addE
-	where
-		addE k = return $ Just $ k
-			{ keyName = keyName k ++ selectExtension (keyFilename source)
-			, keyBackendName = shaNameE size
-			}
+  where
+	addE k = return $ Just $ k
+		{ keyName = keyName k ++ selectExtension (keyFilename source)
+		, keyBackendName = shaNameE size
+		}
 
 selectExtension :: FilePath -> String
 selectExtension f
 	| null es = ""
-	| otherwise = join "." ("":es)
-	where
-		es = filter (not . null) $ reverse $
-			take 2 $ takeWhile shortenough $
-			reverse $ split "." $ takeExtensions f
-		shortenough e
-			| '\n' `elem` e = False -- newline in extension?!
-			| otherwise = length e <= 4 -- long enough for "jpeg"
+	| otherwise = intercalate "." ("":es)
+  where
+	es = filter (not . null) $ reverse $
+		take 2 $ takeWhile shortenough $
+		reverse $ split "." $ filter validExtension $ takeExtensions f
+	shortenough e = length e <= 4 -- long enough for "jpeg"
 
 {- A key's checksum is checked during fsck. -}
 checkKeyChecksum :: SHASize -> Key -> FilePath -> Annex Bool
@@ -137,7 +120,27 @@ checkKeyChecksum size key file = do
 			let filesize = fromIntegral $ fileSize stat
 			check <$> shaN size file filesize
 		_ -> return True
-	where
-		check s
-			| s == dropExtensions (keyName key) = True
-			| otherwise = False
+  where
+	sha = keySha key
+	check s
+		| s == sha = True
+		{- A bug caused checksums to be prefixed with \ in some
+		 - cases; still accept these as legal now that the bug has been
+		 - fixed. -}
+		| '\\' : s == sha = True
+		| otherwise = False
+
+keySha :: Key -> String
+keySha key = dropExtensions (keyName key)
+
+validExtension :: Char -> Bool
+validExtension c
+	| isAlphaNum c = True
+	| c == '.' = True
+	| otherwise = False
+
+{- Upgrade keys that have the \ prefix on their sha due to a bug, or
+ - that contain non-alphanumeric characters in their extension. -}
+needsUpgrade :: Key -> Bool
+needsUpgrade key = "\\" `isPrefixOf` keySha key ||
+	any (not . validExtension) (takeExtensions $ keyName key)
