@@ -17,11 +17,9 @@ module Command (
 	doCommand,
 	whenAnnexed,
 	ifAnnexed,
-	notBareRepo,
 	isBareRepo,
 	numCopies,
-	autoCopies,
-	autoCopiesWith,
+	numCopiesCheck,
 	checkAuto,
 	module ReExported
 ) where
@@ -39,10 +37,9 @@ import Usage as ReExported
 import Logs.Trust
 import Config
 import Annex.CheckAttr
-import qualified Git.Command
 
 {- Generates a normal command -}
-command :: String -> String -> [CommandSeek] -> String -> Command
+command :: String -> String -> [CommandSeek] -> CommandSection -> String -> Command
 command = Command [] Nothing commonChecks False
 
 {- Indicates that a command doesn't need to commit any changes to
@@ -81,17 +78,14 @@ prepCommand Command { cmdseek = seek, cmdcheck = c } params = do
 {- Runs a command through the start, perform and cleanup stages -}
 doCommand :: CommandStart -> CommandCleanup
 doCommand = start
-	where
-		start   = stage $ maybe skip perform
-		perform = stage $ maybe failure cleanup
-		cleanup = stage $ end
-		stage = (=<<)
-		skip = return True
-		failure = showEndFail >> return False
-		end r = do
-			-- zombies from long-running git processes
-			liftIO Git.Command.reap
-			showEndResult r >> return r
+  where
+	start   = stage $ maybe skip perform
+	perform = stage $ maybe failure cleanup
+	cleanup = stage $ status
+	stage = (=<<)
+	skip = return True
+	failure = showEndFail >> return False
+	status r = showEndResult r >> return r
 
 {- Modifies an action to only act on files that are already annexed,
  - and passes the key and backend on to it. -}
@@ -101,47 +95,18 @@ whenAnnexed a file = ifAnnexed file (a file) (return Nothing)
 ifAnnexed :: FilePath -> ((Key, Backend) -> Annex a) -> Annex a -> Annex a
 ifAnnexed file yes no = maybe no yes =<< Backend.lookupFile file
 
-notBareRepo :: Annex a -> Annex a
-notBareRepo a = do
-	whenM isBareRepo $
-		error "You cannot run this subcommand in a bare repository."
-	a
-
 isBareRepo :: Annex Bool
 isBareRepo = fromRepo Git.repoIsLocalBare
 
 numCopies :: FilePath  -> Annex (Maybe Int)
 numCopies file = readish <$> checkAttr "annex.numcopies" file
 
-{- Used for commands that have an auto mode that checks the number of known
- - copies of a key.
- -
- - In auto mode, first checks that the number of known
- - copies of the key is > or < than the numcopies setting, before running
- - the action. Also checks any preferred content settings.
- -}
-autoCopies :: FilePath -> Key -> (Int -> Int -> Bool) -> CommandStart -> CommandStart
-autoCopies file key vs a = Annex.getState Annex.auto >>= go
-	where
-		go False = a
-		go True = do
-			numcopiesattr <- numCopies file
-			needed <- getNumCopies numcopiesattr
-			(_, have) <- trustPartition UnTrusted =<< Remote.keyLocations key
-			if length have `vs` needed then a else stop
-
-autoCopiesWith :: FilePath -> Key -> (Int -> Int -> Bool) -> (Maybe Int -> CommandStart) -> CommandStart
-autoCopiesWith file key vs a = do
+numCopiesCheck :: FilePath -> Key -> (Int -> Int -> Bool) -> Annex Bool
+numCopiesCheck file key vs = do
 	numcopiesattr <- numCopies file
-	Annex.getState Annex.auto >>= auto numcopiesattr
-	where
-		auto numcopiesattr False = a numcopiesattr
-		auto numcopiesattr True = do
-			needed <- getNumCopies numcopiesattr
-			(_, have) <- trustPartition UnTrusted =<< Remote.keyLocations key
-			if length have `vs` needed
-				then a numcopiesattr
-				else stop
+	needed <- getNumCopies numcopiesattr
+	have <- trustExclude UnTrusted =<< Remote.keyLocations key
+	return $ length have `vs` needed
 
 checkAuto :: Annex Bool -> Annex Bool
 checkAuto checker = ifM (Annex.getState Annex.auto)
