@@ -7,14 +7,15 @@
 
 module Command.Fix where
 
+import System.PosixCompat.Files
+
 import Common.Annex
 import Command
 import qualified Annex.Queue
-import Annex.Content
 
 def :: [Command]
-def = [command "fix" paramPaths seek
-	"fix up symlinks to point to annexed content"]
+def = [notDirect $ noCommit $ command "fix" paramPaths seek
+	SectionMaintenance "fix up symlinks to point to annexed content"]
 
 seek :: [CommandSeek]
 seek = [withFilesInGit $ whenAnnexed start]
@@ -22,8 +23,8 @@ seek = [withFilesInGit $ whenAnnexed start]
 {- Fixes the symlink to an annexed file. -}
 start :: FilePath -> (Key, Backend) -> CommandStart
 start file (key, _) = do
-	link <- calcGitLink file key
-	stopUnless ((/=) link <$> liftIO (readSymbolicLink file)) $ do
+	link <- inRepo $ gitAnnexLink file key
+	stopUnless ((/=) (Just link) <$> liftIO (catchMaybeIO $ readSymbolicLink file)) $ do
 		showStart "fix" file
 		next $ perform file link
 
@@ -36,5 +37,5 @@ perform file link = do
 
 cleanup :: FilePath -> CommandCleanup
 cleanup file = do
-	Annex.Queue.add "add" [Param "--force", Param "--"] [file]
+	Annex.Queue.addCommand "add" [Param "--force", Param "--"] [file]
 	return True
