@@ -5,18 +5,23 @@
  - Licensed under the GNU GPL version 3 or higher.
  -}
 
+{-# LANGUAGE CPP #-}
+
 module Command.Fsck where
 
+import System.PosixCompat.Files
+
 import Common.Annex
 import Command
 import qualified Annex
-import qualified Annex.Queue
 import qualified Remote
 import qualified Types.Backend
 import qualified Types.Key
 import qualified Backend
 import Annex.Content
+import Annex.Content.Direct
 import Annex.Perms
+import Annex.Link
 import Logs.Location
 import Logs.Trust
 import Annex.UUID
@@ -26,8 +31,13 @@ import Config
 import qualified Option
 import Types.Key
 import Utility.HumanTime
+import Git.FilePath
 
+#ifndef __WINDOWS__
 import System.Posix.Process (getProcessID)
+#else
+import System.Random (getStdRandom, random)
+#endif
 import Data.Time.Clock.POSIX
 import Data.Time
 import System.Posix.Types (EpochTime)
@@ -35,7 +45,7 @@ import System.Locale
 
 def :: [Command]
 def = [withOptions options $ command "fsck" paramPaths seek
-	"check for problems"]
+	SectionMaintenance "check for problems"]
 
 fromOption :: Option
 fromOption = Option.field ['f'] "from" paramRemote "check remote"
@@ -60,7 +70,7 @@ options =
 
 seek :: [CommandSeek]
 seek =
-	[ withField fromOption Remote.byName $ \from ->
+	[ withField fromOption Remote.byNameWithUUID $ \from ->
 	  withIncremental $ \i -> withFilesInGit $ whenAnnexed $ start from i
 	, withIncremental $ \i -> withBarePresentKeys $ startBare i
 	]
@@ -78,22 +88,22 @@ withIncremental = withValue $ do
 		(True, _, _) ->
 			maybe startIncremental (return . ContIncremental . Just)
 				=<< getStartTime
-	where
-		startIncremental = do
-			recordStartTime
-			return StartIncremental
+  where
+	startIncremental = do
+		recordStartTime
+		return StartIncremental
 
-		checkschedule Nothing = error "bad --incremental-schedule value"
-		checkschedule (Just delta) = do
-			Annex.addCleanup "" $ do
-				v <- getStartTime
-				case v of
-					Nothing -> noop
-					Just started -> do
-						now <- liftIO getPOSIXTime
-						when (now - realToFrac started >= delta) $
-							resetStartTime
-			return True
+	checkschedule Nothing = error "bad --incremental-schedule value"
+	checkschedule (Just delta) = do
+		Annex.addCleanup "" $ do
+			v <- getStartTime
+			case v of
+				Nothing -> noop
+				Just started -> do
+					now <- liftIO getPOSIXTime
+					when (now - realToFrac started >= delta) $
+						resetStartTime
+		return True
 
 start :: Maybe Remote -> Incremental -> FilePath -> (Key, Backend) -> CommandStart
 start from inc file (key, backend) = do
@@ -101,16 +111,17 @@ start from inc file (key, backend) = do
 	case from of
 		Nothing -> go $ perform key file backend numcopies
 		Just r -> go $ performRemote key file backend numcopies r
-	where
-		go = runFsck inc file key
+  where
+	go = runFsck inc file key
 
 perform :: Key -> FilePath -> Backend -> Maybe Int -> Annex Bool
 perform key file backend numcopies = check
 	-- order matters
 	[ fixLink key file
 	, verifyLocationLog key file
+	, verifyDirectMapping key file
 	, checkKeySize key
-	, checkBackend backend key
+	, checkBackend backend key (Just file)
 	, checkKeyNumCopies key file numcopies
 	]
 
@@ -119,48 +130,53 @@ perform key file backend numcopies = check
 performRemote :: Key -> FilePath -> Backend -> Maybe Int -> Remote -> Annex Bool
 performRemote key file backend numcopies remote =
 	dispatch =<< Remote.hasKey remote key
-	where
-		dispatch (Left err) = do
-			showNote err
-			return False
-		dispatch (Right True) = withtmp $ \tmpfile ->
-			ifM (getfile tmpfile)
-				( go True (Just tmpfile)
-				, go True Nothing
-				)
-		dispatch (Right False) = go False Nothing
-		go present localcopy = check
-			[ verifyLocationLogRemote key file remote present
-			, checkKeySizeRemote key remote localcopy
-			, checkBackendRemote backend key remote localcopy
-			, checkKeyNumCopies key file numcopies
-			]
-		withtmp a = do
-			pid <- liftIO getProcessID
-			t <- fromRepo gitAnnexTmpDir
-			createAnnexDirectory t
-			let tmp = t </> "fsck" ++ show pid ++ "." ++ keyFile key
-			let cleanup = liftIO $ catchIO (removeFile tmp) (const noop)
-			cleanup
-			cleanup `after` a tmp
-		getfile tmp =
-			ifM (Remote.retrieveKeyFileCheap remote key tmp)
-				( return True
-				, ifM (Annex.getState Annex.fast)
-					( return False
-					, Remote.retrieveKeyFile remote key Nothing tmp
-					)
+  where
+	dispatch (Left err) = do
+		showNote err
+		return False
+	dispatch (Right True) = withtmp $ \tmpfile ->
+		ifM (getfile tmpfile)
+			( go True (Just tmpfile)
+			, go True Nothing
+			)
+	dispatch (Right False) = go False Nothing
+	go present localcopy = check
+		[ verifyLocationLogRemote key file remote present
+		, checkKeySizeRemote key remote localcopy
+		, checkBackendRemote backend key remote localcopy
+		, checkKeyNumCopies key file numcopies
+		]
+	withtmp a = do
+#ifndef __WINDOWS__
+		v <- liftIO getProcessID
+#else
+		v <- liftIO (getStdRandom random :: IO Int)
+#endif
+		t <- fromRepo gitAnnexTmpDir
+		createAnnexDirectory t
+		let tmp = t </> "fsck" ++ show v ++ "." ++ keyFile key
+		let cleanup = liftIO $ catchIO (removeFile tmp) (const noop)
+		cleanup
+		cleanup `after` a tmp
+	getfile tmp =
+		ifM (Remote.retrieveKeyFileCheap remote key tmp)
+			( return True
+			, ifM (Annex.getState Annex.fast)
+				( return False
+				, Remote.retrieveKeyFile remote key Nothing tmp dummymeter
 				)
+			)
+	dummymeter _ = noop
 
 {- To fsck a bare repository, fsck each key in the location log. -}
 withBarePresentKeys :: (Key -> CommandStart) -> CommandSeek
 withBarePresentKeys a params = isBareRepo >>= go
-	where
-		go False = return []
-		go True = do
-			unless (null params) $
-				error "fsck should be run without parameters in a bare repository"
-			map a <$> loggedKeys
+  where
+	go False = return []
+	go True = do
+		unless (null params) $
+			error "fsck should be run without parameters in a bare repository"
+		map a <$> loggedKeys
 
 startBare :: Incremental -> Key -> CommandStart
 startBare inc key = case Backend.maybeLookupBackendName (Types.Key.keyBackendName key) of
@@ -174,52 +190,51 @@ performBare :: Key -> Backend -> Annex Bool
 performBare key backend = check
 	[ verifyLocationLog key (key2file key)
 	, checkKeySize key
-	, checkBackend backend key
+	, checkBackend backend key Nothing
 	]
 
 check :: [Annex Bool] -> Annex Bool
 check cs = all id <$> sequence cs
 
-{- Checks that the file's symlink points correctly to the content. -}
+{- Checks that the file's link points correctly to the content.
+ -
+ - In direct mode, there is only a link when the content is not present.
+ -}
 fixLink :: Key -> FilePath -> Annex Bool
 fixLink key file = do
-	want <- calcGitLink file key
-	have <- liftIO $ readSymbolicLink file
-	when (want /= have) $ do
-		{- Version 3.20120227 had a bug that could cause content
-		 - to be stored in the wrong hash directory. Clean up
-		 - after the bug by moving the content.
-		 -}
-		whenM (liftIO $ doesFileExist file) $
-			unlessM (inAnnex key) $ do
-				showNote "fixing content location"
-				dir <- liftIO $ parentDir <$> absPath file
-				let content = absPathFrom dir have
-				liftIO $ allowWrite (parentDir content)
-				moveAnnex key content
-
-		showNote "fixing link"
-		liftIO $ createDirectoryIfMissing True (parentDir file)
-		liftIO $ removeFile file
-		liftIO $ createSymbolicLink want file
-		Annex.Queue.addCommand "add" [Param "--force", Param "--"] [file]
+	want <- inRepo $ gitAnnexLink file key
+	have <- getAnnexLinkTarget file
+	maybe noop (go want) have
 	return True
+  where
+	go want have
+		| want /= fromInternalGitPath have = do
+			showNote "fixing link"
+			liftIO $ createDirectoryIfMissing True (parentDir file)
+			liftIO $ removeFile file
+			addAnnexLink want file
+		| otherwise = noop
 
 {- Checks that the location log reflects the current status of the key,
-   in this repository only. -}
+ - in this repository only. -}
 verifyLocationLog :: Key -> String -> Annex Bool
 verifyLocationLog key desc = do
 	present <- inAnnex key
+	direct <- isDirect
+	u <- getUUID
 	
-	-- Since we're checking that a key's file is present, throw
-	-- in a permission fixup here too.
-	when present $ do
-		file <- inRepo $ gitAnnexLocation key
+	{- Since we're checking that a key's file is present, throw
+	 - in a permission fixup here too. -}
+	when (present && not direct) $ do
+		file <- calcRepo $ gitAnnexLocation key
 		freezeContent file
 		freezeContentDir file
 
-	u <- getUUID
-	verifyLocationLog' key desc present u (logChange key u)
+	{- In direct mode, modified files will show up as not present,
+	 - but that is expected and not something to do anything about. -}
+	if (direct && not present)
+		then return True
+		else verifyLocationLog' key desc present u (logChange key u)
 
 verifyLocationLogRemote :: Key -> String -> Remote -> Bool -> Annex Bool
 verifyLocationLogRemote key desc remote present =
@@ -242,20 +257,39 @@ verifyLocationLog' key desc present u bad = do
 					"but its content is missing."
 				return False
 		_ -> return True
-	where
-		fix s = do
-			showNote "fixing location log"
-			bad s
+  where
+	fix s = do
+		showNote "fixing location log"
+		bad s
+
+{- Ensures the direct mode mapping file is consistent. Each file
+ - it lists for the key should exist, and the specified file should be
+ - included in it.
+ -}
+verifyDirectMapping :: Key -> FilePath -> Annex Bool
+verifyDirectMapping key file = do
+	whenM isDirect $ do
+		fs <- addAssociatedFile key file
+		forM_ fs $ \f -> 
+			unlessM (liftIO $ doesFileExist f) $
+				void $ removeAssociatedFile key f
+	return True
 
 {- The size of the data for a key is checked against the size encoded in
- - the key's metadata, if available. -}
+ - the key's metadata, if available.
+ -
+ - Not checked in direct mode, because files can be changed directly.
+  -}
 checkKeySize :: Key -> Annex Bool
-checkKeySize key = do
-	file <- inRepo $ gitAnnexLocation key
-	ifM (liftIO $ doesFileExist file)
-		( checkKeySizeOr badContent key file
-		, return True
-		)
+checkKeySize key = ifM isDirect
+	( return True
+	, do
+		file <- calcRepo $ gitAnnexLocation key
+		ifM (liftIO $ doesFileExist file)
+			( checkKeySizeOr badContent key file
+			, return True
+			)
+	)
 
 checkKeySizeRemote :: Key -> Remote -> Maybe FilePath -> Annex Bool
 checkKeySizeRemote _ _ Nothing = return True
@@ -269,40 +303,63 @@ checkKeySizeOr bad key file = case Types.Key.keySize key of
 		size' <- fromIntegral . fileSize
 			<$> liftIO (getFileStatus file)
 		comparesizes size size'
-	where
-		comparesizes a b = do
-			let same = a == b
-			unless same $ badsize a b
-			return same
-		badsize a b = do
-			msg <- bad key
-			warning $ concat
-				[ "Bad file size ("
-				, compareSizes storageUnits True a b
-				, "); "
-				, msg
-				]
+  where
+	comparesizes a b = do
+		let same = a == b
+		unless same $ badsize a b
+		return same
+	badsize a b = do
+		msg <- bad key
+		warning $ concat
+			[ "Bad file size ("
+			, compareSizes storageUnits True a b
+			, "); "
+			, msg
+			]
 
-checkBackend :: Backend -> Key -> Annex Bool
-checkBackend backend key = do
-	file <- inRepo (gitAnnexLocation key)
-	checkBackendOr badContent backend key file
+{- Runs the backend specific check on a key's content.
+ -
+ - In direct mode this is not done if the file has clearly been modified,
+ - because modification of direct mode files is allowed. It's still done
+ - if the file does not appear modified, to catch disk corruption, etc.
+ -}
+checkBackend :: Backend -> Key -> Maybe FilePath -> Annex Bool
+checkBackend backend key mfile = go =<< isDirect
+  where
+  	go False = do
+		content <- calcRepo $ gitAnnexLocation key
+		checkBackendOr badContent backend key content
+	go True = maybe nocheck checkdirect mfile
+	checkdirect file = ifM (goodContent key file)
+		( checkBackendOr' (badContentDirect file) backend key file
+			(goodContent key file)
+		, nocheck
+		)
+	nocheck = return True
 
 checkBackendRemote :: Backend -> Key -> Remote -> Maybe FilePath -> Annex Bool
 checkBackendRemote backend key remote = maybe (return True) go
-	where
-		go = checkBackendOr (badContentRemote remote) backend key
+  where
+	go file = checkBackendOr (badContentRemote remote) backend key file
 
 checkBackendOr :: (Key -> Annex String) -> Backend -> Key -> FilePath -> Annex Bool
 checkBackendOr bad backend key file =
+	checkBackendOr' bad backend key file (return True)
+
+checkBackendOr' :: (Key -> Annex String) -> Backend -> Key -> FilePath -> Annex Bool -> Annex Bool
+checkBackendOr' bad backend key file postcheck =
 	case Types.Backend.fsckKey backend of
 		Nothing -> return True
 		Just a -> do
 			ok <- a key file
-			unless ok $ do
-				msg <- bad key
-				warning $ "Bad file content; " ++ msg
-			return ok
+			ifM postcheck
+				( do
+					unless ok $ do
+						msg <- bad key
+						warning $ "Bad file content; " ++ msg
+					return ok
+				, return True
+				)
 
 checkKeyNumCopies :: Key -> FilePath -> Maybe Int -> Annex Bool
 checkKeyNumCopies key file numcopies = do
@@ -338,13 +395,19 @@ badContent key = do
 	dest <- moveBad key
 	return $ "moved to " ++ dest
 
+{- Bad content is left where it is, but we touch the file, so it'll be
+ - committed to a new key. -}
+badContentDirect :: FilePath -> Key -> Annex String
+badContentDirect file key = do
+	void $ liftIO $ catchMaybeIO $ touchFile file
+	logStatus key InfoMissing
+	return $ "left in place for you to examine"
+
 badContentRemote :: Remote -> Key -> Annex String
 badContentRemote remote key = do
 	ok <- Remote.removeKey remote key
-	-- better safe than sorry: assume the remote dropped the key
-	-- even if it seemed to fail; the failure could have occurred
-	-- after it really dropped it
-	Remote.logStatus remote key InfoMissing
+	when ok $
+		Remote.logStatus remote key InfoMissing
 	return $ (if ok then "dropped from " else "failed to drop from ")
 		++ Remote.name remote
 
@@ -385,21 +448,23 @@ needFsck _ _ = return True
  -}
 recordFsckTime :: Key -> Annex ()
 recordFsckTime key = do
-	parent <- parentDir <$> inRepo (gitAnnexLocation key)
+	parent <- parentDir <$> calcRepo (gitAnnexLocation key)
 	liftIO $ void $ tryIO $ do
 		touchFile parent
+#ifndef __WINDOWS__
 		setSticky parent
+#endif
 
 getFsckTime :: Key -> Annex (Maybe EpochTime)
 getFsckTime key = do
-	parent <- parentDir <$> inRepo (gitAnnexLocation key)
+	parent <- parentDir <$> calcRepo (gitAnnexLocation key)
 	liftIO $ catchDefaultIO Nothing $ do
 		s <- getFileStatus parent
 		return $ if isSticky $ fileMode s
 			then Just $ modificationTime s
 			else Nothing
 
-{- Records the start time of an interactive fsck.
+{- Records the start time of an incremental fsck.
  -
  - To guard against time stamp damange (for example, if an annex directory
  - is copied without -a), the fsckstate file contains a time that should
@@ -414,9 +479,9 @@ recordStartTime = do
 		t <- modificationTime <$> getFileStatus f
 		hPutStr h $ showTime $ realToFrac t
 		hClose h
-	where
-		showTime :: POSIXTime -> String
-		showTime = show
+  where
+	showTime :: POSIXTime -> String
+	showTime = show
 
 resetStartTime :: Annex ()
 resetStartTime = liftIO . nukeFile =<< fromRepo gitAnnexFsckState
@@ -431,7 +496,7 @@ getStartTime = do
 		return $ if Just (realToFrac timestamp) == t
 			then Just timestamp
 			else Nothing
-	where
-		readishTime :: String -> Maybe POSIXTime
-		readishTime s = utcTimeToPOSIXSeconds <$>
-			parseTime defaultTimeLocale "%s%Qs" s
+  where
+	readishTime :: String -> Maybe POSIXTime
+	readishTime s = utcTimeToPOSIXSeconds <$>
+		parseTime defaultTimeLocale "%s%Qs" s
