@@ -12,8 +12,8 @@ import Command
 import qualified Annex.Branch
 
 def :: [Command]
-def = [command "merge" paramNothing seek
-		"auto-merge remote changes into git-annex branch"]
+def = [command "merge" paramNothing seek SectionMaintenance
+	"auto-merge remote changes into git-annex branch"]
 
 seek :: [CommandSeek]
 seek = [withNothing start]
@@ -26,4 +26,6 @@ start = do
 perform :: CommandPerform
 perform = do
 	Annex.Branch.update
+	-- commit explicitly, in case no remote branches were merged
+	Annex.Branch.commit "update"
 	next $ return True
