@@ -14,24 +14,26 @@ import Types.Key
 import Annex.Content
 import qualified Command.Add
 import Logs.Web
+import Logs.Location
+import Utility.CopyFile
 
 def :: [Command]
-def = [command "rekey"
+def = [notDirect $ command "rekey"
 	(paramOptional $ paramRepeating $ paramPair paramPath paramKey)
-	seek "change keys used for files"]
+	seek SectionPlumbing "change keys used for files"]
 
 seek :: [CommandSeek]
 seek = [withPairs start]
 
 start :: (FilePath, String) -> CommandStart
 start (file, keyname) = ifAnnexed file go stop
-	where
-		newkey = fromMaybe (error "bad key") $ file2key keyname
-		go (oldkey, _)
-			| oldkey == newkey = stop
-			| otherwise = do
-				showStart "rekey" file
-				next $ perform file oldkey newkey
+  where
+	newkey = fromMaybe (error "bad key") $ file2key keyname
+	go (oldkey, _)
+		| oldkey == newkey = stop
+		| otherwise = do
+			showStart "rekey" file
+			next $ perform file oldkey newkey
 
 perform :: FilePath -> Key -> Key -> CommandPerform
 perform file oldkey newkey = do
@@ -44,12 +46,15 @@ perform file oldkey newkey = do
 			return True
 	next $ cleanup file oldkey newkey
 
-{- Make a hard link to the old key content, to avoid wasting disk space. -}
+{- Make a hard link to the old key content (when supported),
+ - to avoid wasting disk space. -}
 linkKey :: Key -> Key -> Annex Bool
 linkKey oldkey newkey = getViaTmpUnchecked newkey $ \tmp -> do
-	src <- inRepo $ gitAnnexLocation oldkey
-	liftIO $ unlessM (doesFileExist tmp) $ createLink src tmp
-	return True
+	src <- calcRepo $ gitAnnexLocation oldkey
+	liftIO $ ifM (doesFileExist tmp)
+		( return True
+		, createLinkOrCopy src tmp
+		)
 
 cleanup :: FilePath -> Key -> Key -> CommandCleanup
 cleanup file oldkey newkey = do
@@ -61,4 +66,6 @@ cleanup file oldkey newkey = do
 
 	-- Update symlink to use the new key.
 	liftIO $ removeFile file
-	Command.Add.cleanup file newkey True
+	Command.Add.addLink file newkey True
+	logStatus newkey InfoPresent
+	return True
