@@ -11,15 +11,14 @@ import Common.Annex
 import Command
 import Annex.Content
 import Utility.CopyFile
-import Utility.FileMode
 
 def :: [Command]
 def =
 	[ c "unlock" "unlock files for modification"
 	, c "edit" "same as unlock"
 	]
-	where
-		c n = command n paramPaths seek
+  where
+	c n = notDirect . command n paramPaths seek SectionCommon
 
 seek :: [CommandSeek]
 seek = [withFilesInGit $ whenAnnexed start]
@@ -34,19 +33,18 @@ start file (key, _) = do
 perform :: FilePath -> Key -> CommandPerform
 perform dest key = do
 	unlessM (inAnnex key) $ error "content not present"
-	
-	checkDiskSpace key
+	unlessM (checkDiskSpace Nothing key 0) $ error "cannot unlock"
 
-	src <- inRepo $ gitAnnexLocation key
+	src <- calcRepo $ gitAnnexLocation key
 	tmpdest <- fromRepo $ gitAnnexTmpLocation key
 	liftIO $ createDirectoryIfMissing True (parentDir tmpdest)
 	showAction "copying"
-	ok <- liftIO $ copyFileExternal src tmpdest
-        if ok
-                then do
+	ifM (liftIO $ copyFileExternal src tmpdest)
+		( do
 			liftIO $ do
 				removeFile dest
 				moveFile tmpdest dest
-				allowWrite dest
+			thawContent dest
 			next $ return True
-                else error "copy failed!"
+		, error "copy failed!"
+		)
