@@ -13,9 +13,13 @@ import qualified Git
 import qualified Git.Construct
 import Annex.Content
 import Config
+import Config.Cost
 import Logs.Web
 import qualified Utility.Url as Url
 import Types.Key
+import Utility.Metered
+
+import qualified Data.Map as M
 
 remote :: RemoteType
 remote = RemoteType {
@@ -33,8 +37,8 @@ list = do
 	r <- liftIO $ Git.Construct.remoteNamed "web" Git.Construct.fromUnknown
 	return [r]
 
-gen :: Git.Repo -> UUID -> Maybe RemoteConfig -> Annex Remote
-gen r _ _ = 
+gen :: Git.Repo -> UUID -> RemoteConfig -> RemoteGitConfig -> Annex Remote
+gen r _ _ gc = 
 	return Remote {
 		uuid = webUUID,
 		cost = expensiveRemoteCost,
@@ -46,22 +50,24 @@ gen r _ _ =
 		hasKey = checkKey,
 		hasKeyCheap = False,
 		whereisKey = Just getUrls,
-		config = Nothing,
+		config = M.empty,
+		gitconfig = gc,
 		localpath = Nothing,
 		repo = r,
 		readonly = True,
+		globallyAvailable = True,
 		remotetype = remote
 	}
 
-downloadKey :: Key -> AssociatedFile -> FilePath -> Annex Bool
-downloadKey key _file dest = get =<< getUrls key
-	where
-		get [] = do
-			warning "no known url"
-			return False
-		get urls = do
-			showOutput -- make way for download progress bar
-			downloadUrl urls dest
+downloadKey :: Key -> AssociatedFile -> FilePath -> MeterUpdate -> Annex Bool
+downloadKey key _file dest _p = get =<< getUrls key
+  where
+	get [] = do
+		warning "no known url"
+		return False
+	get urls = do
+		showOutput -- make way for download progress bar
+		downloadUrl urls dest
 
 downloadKeyCheap :: Key -> FilePath -> Annex Bool
 downloadKeyCheap _ _ = return False
@@ -72,9 +78,9 @@ uploadKey _ _ _ = do
 	return False
 
 dropKey :: Key -> Annex Bool
-dropKey _ = do
-	warning "removal from web not supported"
-	return False
+dropKey k = do
+	mapM_ (setUrlMissing k) =<< getUrls k
+	return True
 
 checkKey :: Key -> Annex (Either String Bool)
 checkKey key = do
